import { Utilities } from '@/util/Utilities';
import {
    Modal, ModalService, Select, Text, TextInput, Number, Form, LoadingService, DialogService,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Sm00005000TabParametrizacoesIndicesPadroesController {
    private basicModal!: Modal;

    public somaPercentualGrid: any = {
        somaPercentual: null,
    };

    public async exibirModalIndices() {
        this.basicModal = await ModalService.createFromJson('sm_00005_000_parametrizacoes_services/modal/sm_00005_000_parametrizacoes_indices_padroes_modal', true);
        this.basicModal.show();
    }

    public fecharModalIndices() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    public async abrirModalIndices() {
        await this.exibirModalIndices();
        Metadata.getInstance<Text>('titleText').text = 'Novo Índice Financeiro Padrão';
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-indices');
        let somaPercentual = 0;
        grid.datasource.data.forEach((value) => {
            const nrpercentindice = value.NRPERCENTINDICE;
            somaPercentual += nrpercentindice;
        });
        this.somaPercentualGrid.somaPercentual = somaPercentual;
        if (somaPercentual <= 100) {
            const valorColuna = 100 - somaPercentual;
            Metadata.getInstance<Number>('NRPERCENTINDICE').value = valorColuna;
        }
    }

    public async abrirModalEdicaoIndices({ row }: any) {
        await this.exibirModalIndices();
        Metadata.getInstance<Text>('titleText').text = 'Editar Índice Financeiro Padrão';
        Metadata.getInstance<Select>('CDINDICE').value = row.CDINDICE;
        Metadata.getInstance<TextInput>('NRPERCENTINDICE').value = row.NRPERCENTINDICE;
        Metadata.getInstance<TextInput>('NRINDICESPADROES').value = row.NRINDICESPADROES;
        Metadata.getInstance<Select>('CDINDICE').readonly = true;
    }

    public saveIndicesFinanceirosPadrao() {
        const somaValorInput = Metadata.getInstance<Number>('NRPERCENTINDICE').value + this.somaPercentualGrid.somaPercentual;
        if (Metadata.getInstance<TextInput>('NRINDICESPADROES').value == null && somaValorInput > 100) {
            Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'A soma dos percentuais de utilização dos índices financeiros ultrapassou o total de 100%.');
        } else if (Metadata.getInstance<TextInput>('NRINDICESPADROES').value != null && Metadata.getInstance<Number>('NRPERCENTINDICE').value > this.somaPercentualGrid.somaPercentual) {
            Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'A soma dos percentuais de utilização dos índices financeiros ultrapassou o total de 100%.');
        } else {
            const form = Metadata.getInstance<Form>('form_modal_indices');
            const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-indices');
            if (Utilities.validateForm(form)) {
                LoadingService.show();
                Http.post('/saveIndicesFinanceirosPadrao', {
                    NRINDICESPADROES: Metadata.getInstance<TextInput>('NRINDICESPADROES').value,
                    CDINDICE: Metadata.getInstance<Select>('CDINDICE').value,
                    NRPERCENTINDICE: Metadata.getInstance<Number>('NRPERCENTINDICE').value,
                    CDTPINDICE: Metadata.getInstance<Select>('CDINDICE').selectValue.CDTPINDICE,
                    IDPERVALOR: Metadata.getInstance<Select>('CDINDICE').selectValue.IDPERVALOR,
                }).then(() => {
                    this.fecharModalIndices();
                    grid.datasource.get();
                    Utilities.showSuccessMessage();
                }).catch((error) => {
                    if (error.response.data.error === 'EXISTE') {
                        Utilities.showDialogMessage('error', 'Operação bloqueada!', 'O índice selecionado já está inserido.');
                    } else {
                        Utilities.showErrorMessage(error);
                    }
                }).finally(() => {
                    LoadingService.hide();
                });
            }
        }
    }

    public showConfirmDeleteMessageIndices({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteParametrosIndice();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteParametrosIndice() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-indices');
        LoadingService.show();
        Http.post('/deleteParametrosIndice', grid.selectedRows)
            .then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }
}
